/**
 * Test case for create.
 * Runs with mocha.
 */
'use strict'

const create = require('../lib/create.js')
const apemanport = require('apemanport')
const apemanrequest = require('apemanrequest')
const co = require('co')
const apemanApp = require('apeman-app')
const http = require('http')
const assert = require('assert')

describe('create', () => {
  let proxyServer, proxyBaseUrl,
    targetServer, targetBaseUrl

  let request = apemanrequest.create({})
  before(() => co(function * () {
    let port = yield apemanport.find()
    targetBaseUrl = `http://localhost:${port}/`
    targetServer = http.createServer((req, res) => {
      res.writeHead(200, { 'Content-Type': 'text/plain' })
      res.write('request successfully proxied!')
      res.end()
    }).listen(port)
  }))

  before(() => co(function * () {
    let app = create({
      target: targetBaseUrl
    })
    assert.ok(app)
    let port = yield apemanport.find()
    proxyServer = yield apemanApp('testing-app', {
      port,
      configuration: {
        $apps: {
          'testing-app': {
            '*': [
              app
            ],
            '/foo/bar': [
              (ctx) => {
                console.log('!!!ctx', ctx.url)
              }
            ]
          }
        }
      }
    })
    proxyBaseUrl = `http://localhost:${port}`
  }))

  after(() => co(function * () {
    yield proxyServer.close()
  }))

  after(() => co(function * () {
    targetServer.close()
  }))

  it('Create', () => co(function * () {
    let { body, statusCode } = yield request({
      url: `${proxyBaseUrl}`,
      method: 'GET'
    })
    assert.equal(statusCode, 200)
    assert.equal(body, 'request successfully proxied!')
  }))
})

/* global describe, before, after, it */
